/*
 * Egyszeru AVR demonstracio. Vezereljuk a LED-et, mely OC1A labtol kozvetlenul
 * foldre (GND) csatoljuk. A LED fenyerejet impulzus-szelesseg modulacioval (PWM)
 * vezereljuk. Minden egyes PWM periodus utan, a PWM erteke novelodik, vagy
 * csokken 1-el.
*/

#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>

#include "pwm_def.h"

enum { UP, DOWN };

ISR (TIMER1_OVF_vect)
{
  static uint16_t pwm;
  static uint8_t direction;

  switch (direction)
  {
    case UP:   if (++pwm == TIMER1_TOP)
                 direction = DOWN;
               break;

    case DOWN: if (--pwm == 0)
                 direction = UP;
               break;
  }

  OCR = pwm;
}

void ioinit (void)
{
  //A Timer 1 10-bites fast PWM-et biztosit ATMEGA128, AT90CAN128,
  // ATMEGA8, ATMEGA16, ATMEGA32 MCU eseteben
  TCCR1A |= TIMER1_PWM_INIT_A;
  TCCR1B |= TIMER1_PWM_INIT_B;
    
  //Elinditjuk a timer 1-et.
  TCCR1B |= TIMER1_CLOCKSOURCE;

  //A PWM erteket 0-ra allitjuk.
  OCR = 0;

  //OC1-et (OC1A = PB5 = 5) kimentetkent engedelyezzuk.
  DDROC = _BV(OC1); //DDROC = DDRB

  //Engedelyezzuk a timer 1 tulcsordulasi megszakitasat.
  TIMSK = _BV(TOIE1);

  sei();
}

int main (void) {
  
  ioinit();

  //Vegtelen ciklus, a megszakitasok vegzik a PWM-jel formalasat.

  for (;;) sleep_mode();

  return (0);
}
